#!/usr/bin/env python3
from io import StringIO

from gentokenlookup import gentokenlookup

# copied from llhttp.h, and stripped trailing spaces and backslashes.
SRC = '''
  XX(0, DELETE, DELETE)
  XX(1, GET, GET)
  XX(2, HEAD, HEAD)
  XX(3, POST, POST)
  XX(4, PUT, PUT)
  XX(5, CONNECT, CONNECT)
  XX(6, OPTIONS, OPTIONS)
  XX(7, TRACE, TRACE)
  XX(8, COPY, COPY)
  XX(9, LOCK, LOCK)
  XX(10, MKCOL, MKCOL)
  XX(11, MOVE, MOVE)
  XX(12, PROPFIND, PROPFIND)
  XX(13, PROPPATCH, PROPPATCH)
  XX(14, SEARCH, SEARCH)
  XX(15, UNLOCK, UNLOCK)
  XX(16, BIND, BIND)
  XX(17, REBIND, REBIND)
  XX(18, UNBIND, UNBIND)
  XX(19, ACL, ACL)
  XX(20, REPORT, REPORT)
  XX(21, MKACTIVITY, MKACTIVITY)
  XX(22, CHECKOUT, CHECKOUT)
  XX(23, MERGE, MERGE)
  XX(24, MSEARCH, M-SEARCH)
  XX(25, NOTIFY, NOTIFY)
  XX(26, SUBSCRIBE, SUBSCRIBE)
  XX(27, UNSUBSCRIBE, UNSUBSCRIBE)
  XX(28, PATCH, PATCH)
  XX(29, PURGE, PURGE)
  XX(30, MKCALENDAR, MKCALENDAR)
  XX(31, LINK, LINK)
  XX(32, UNLINK, UNLINK)
  XX(33, SOURCE, SOURCE)
'''

if __name__ == '__main__':
    methods = []
    for line in StringIO(SRC):
        line = line.strip()
        if not line.startswith('XX'):
            continue
        _, m, _ = line.split(',', 2)
        methods.append(m.strip())
    gentokenlookup(methods, 'HTTP_')



#!/usr/bin/env python3
# -*- coding: utf-8 -*-

# This scripts reads static table entries [1] and generates
# nghttp2_hd_static_entry table.  This table is used in
# lib/nghttp2_hd.c.
#
# [1] https://httpwg.org/specs/rfc7541.html

import re, sys

def hd_map_hash(name):
  h = 2166136261

  # FNV hash variant: http://isthe.com/chongo/tech/comp/fnv/
  for c in name:
    h ^= ord(c)
    h *= 16777619
    h &= 0xffffffff

  return h

entries = []
for line in sys.stdin:
    m = re.match(r'(\d+)\s+(\S+)\s+(\S.*)?', line)
    val = m.group(3).strip() if m.group(3) else ''
    entries.append((int(m.group(1)), m.group(2), val))

print('static nghttp2_hd_entry static_table[] = {')
idx = 0
for i, ent in enumerate(entries):
    if entries[idx][1] != ent[1]:
        idx = i
    print('MAKE_STATIC_ENT("{}", "{}", {}, {}u),'\
        .format(ent[1], ent[2], entries[idx][0] - 1, hd_map_hash(ent[1])))
print('};')



#!/usr/bin/env python3

# script to extract commit author's name from standard input.  The
# input should be <AUTHOR>:<EMAIL>, one per line.
# This script expects the input is created by git-log command:
#
#   git log --format=%aN:%aE
#
# This script removes duplicates based on email address, breaking a
# tie with longer author name.  Among the all author names extract the
# previous step, we remove duplicate by case-insensitive match.
#
# So we can do this in one line:
#
#   git log --format=%aN:%aE | sort | uniq | ./author.py > authors

import sys

edict = {}

for line in sys.stdin:
    author, email = line.strip().split(':', 1)
    if email in edict:
        an = edict[email]
        if len(an) < len(author) or an > author:
            sys.stderr.write(
                'eliminated {} in favor of {}\n'.format(an, author))
            edict[email] = author
        else:
            sys.stderr.write(
                'eliminated {} in favor of {}\n'.format(author, an))
    else:
        edict[email] = author

names = list(sorted(edict.values()))

ndict = {}

for name in names:
    lowname = name.lower()
    if lowname in ndict:
        an = ndict[lowname]
        if an > name:
            sys.stderr.write('eliminated {} in favor of {}\n'.format(an, name))
            ndict[lowname] = name
        else:
            sys.stderr.write('eliminated {} in favor of {}\n'.format(name, an))
    else:
        ndict[lowname] = name

for name in sorted(ndict.values()):
    print(name)



#!/usr/bin/env python3

from gentokenlookup import gentokenlookup

OPTIONS = [
    "private-key-file",
    "private-key-passwd-file",
    "certificate-file",
    "dh-param-file",
    "subcert",
    "backend",
    "frontend",
    "workers",
    "http2-max-concurrent-streams",
    "log-level",
    "daemon",
    "http2-proxy",
    "http2-bridge",
    "client-proxy",
    "add-x-forwarded-for",
    "strip-incoming-x-forwarded-for",
    "no-via",
    "frontend-http2-read-timeout",
    "frontend-read-timeout",
    "frontend-write-timeout",
    "backend-read-timeout",
    "backend-write-timeout",
    "stream-read-timeout",
    "stream-write-timeout",
    "accesslog-file",
    "accesslog-syslog",
    "accesslog-format",
    "errorlog-file",
    "errorlog-syslog",
    "backend-keep-alive-timeout",
    "frontend-http2-window-bits",
    "backend-http2-window-bits",
    "frontend-http2-connection-window-bits",
    "backend-http2-connection-window-bits",
    "frontend-no-tls",
    "backend-no-tls",
    "backend-tls-sni-field",
    "pid-file",
    "user",
    "syslog-facility",
    "backlog",
    "ciphers",
    "client",
    "insecure",
    "cacert",
    "backend-ipv4",
    "backend-ipv6",
    "backend-http-proxy-uri",
    "read-rate",
    "read-burst",
    "write-rate",
    "write-burst",
    "worker-read-rate",
    "worker-read-burst",
    "worker-write-rate",
    "worker-write-burst",
    "npn-list",
    "tls-proto-list",
    "verify-client",
    "verify-client-cacert",
    "client-private-key-file",
    "client-cert-file",
    "frontend-http2-dump-request-header",
    "frontend-http2-dump-response-header",
    "http2-no-cookie-crumbling",
    "frontend-frame-debug",
    "padding",
    "altsvc",
    "add-request-header",
    "add-response-header",
    "worker-frontend-connections",
    "no-location-rewrite",
    "no-host-rewrite",
    "backend-http1-connections-per-host",
    "backend-http1-connections-per-frontend",
    "listener-disable-timeout",
    "tls-ticket-key-file",
    "rlimit-nofile",
    "backend-request-buffer",
    "backend-response-buffer",
    "no-server-push",
    "backend-http2-connections-per-worker",
    "fetch-ocsp-response-file",
    "ocsp-update-interval",
    "no-ocsp",
    "include",
    "tls-ticket-key-cipher",
    "host-rewrite",
    "tls-session-cache-memcached",
    "tls-session-cache-memcached-tls",
    "tls-ticket-key-memcached",
    "tls-ticket-key-memcached-interval",
    "tls-ticket-key-memcached-max-retry",
    "tls-ticket-key-memcached-max-fail",
    "mruby-file",
    "accept-proxy-protocol",
    "conf",
    "fastopen",
    "tls-dyn-rec-warmup-threshold",
    "tls-dyn-rec-idle-timeout",
    "add-forwarded",
    "strip-incoming-forwarded",
    "forwarded-by",
    "forwarded-for",
    "response-header-field-buffer",
    "max-response-header-fields",
    "request-header-field-buffer",
    "max-request-header-fields",
    "header-field-buffer",
    "max-header-fields",
    "no-http2-cipher-block-list",
    "no-http2-cipher-black-list",
    "backend-http1-tls",
    "tls-session-cache-memcached-cert-file",
    "tls-session-cache-memcached-private-key-file",
    "tls-session-cache-memcached-address-family",
    "tls-ticket-key-memcached-tls",
    "tls-ticket-key-memcached-cert-file",
    "tls-ticket-key-memcached-private-key-file",
    "tls-ticket-key-memcached-address-family",
    "backend-address-family",
    "frontend-http2-max-concurrent-streams",
    "backend-http2-max-concurrent-streams",
    "backend-connections-per-frontend",
    "backend-tls",
    "backend-connections-per-host",
    "error-page",
    "no-kqueue",
    "frontend-http2-settings-timeout",
    "backend-http2-settings-timeout",
    "api-max-request-body",
    "backend-max-backoff",
    "server-name",
    "no-server-rewrite",
    "frontend-http2-optimize-write-buffer-size",
    "frontend-http2-optimize-window-size",
    "frontend-http2-window-size",
    "frontend-http2-connection-window-size",
    "backend-http2-window-size",
    "backend-http2-connection-window-size",
    "frontend-http2-encoder-dynamic-table-size",
    "frontend-http2-decoder-dynamic-table-size",
    "backend-http2-encoder-dynamic-table-size",
    "backend-http2-decoder-dynamic-table-size",
    "ecdh-curves",
    "tls-sct-dir",
    "backend-connect-timeout",
    "dns-cache-timeout",
    "dns-lookup-timeout",
    "dns-max-try",
    "frontend-keep-alive-timeout",
    "psk-secrets",
    "client-psk-secrets",
    "client-no-http2-cipher-block-list",
    "client-no-http2-cipher-black-list",
    "client-ciphers",
    "accesslog-write-early",
    "tls-min-proto-version",
    "tls-max-proto-version",
    "redirect-https-port",
    "frontend-max-requests",
    "single-thread",
    "single-process",
    "no-add-x-forwarded-proto",
    "no-strip-incoming-x-forwarded-proto",
    "ocsp-startup",
    "no-verify-ocsp",
    "verify-client-tolerate-expired",
    "ignore-per-pattern-mruby-error",
    "tls-no-postpone-early-data",
    "tls-max-early-data",
    "tls13-ciphers",
    "tls13-client-ciphers",
    "no-strip-incoming-early-data",
    "quic-bpf-program-file",
    "no-quic-bpf",
    "http2-altsvc",
    "frontend-http3-read-timeout",
    "frontend-quic-idle-timeout",
    "frontend-quic-debug-log",
    "frontend-http3-window-size",
    "frontend-http3-connection-window-size",
    "frontend-http3-max-window-size",
    "frontend-http3-max-connection-window-size",
    "frontend-http3-max-concurrent-streams",
    "frontend-quic-early-data",
    "frontend-quic-qlog-dir",
    "frontend-quic-require-token",
    "frontend-quic-congestion-controller",
    "quic-server-id",
    "frontend-quic-secret-file",
    "rlimit-memlock",
    "max-worker-processes",
    "worker-process-grace-shutdown-period",
    "frontend-quic-initial-rtt",
    "require-http-scheme",
    "tls-ktls",
]

LOGVARS = [
    "remote_addr",
    "time_local",
    "time_iso8601",
    "request",
    "status",
    "body_bytes_sent",
    "remote_port",
    "server_port",
    "request_time",
    "pid",
    "alpn",
    "ssl_cipher",
    "ssl_protocol",
    "ssl_session_id",
    "ssl_session_reused",
    "tls_cipher",
    "tls_protocol",
    "tls_session_id",
    "tls_session_reused",
    "tls_sni",
    "tls_client_fingerprint_sha256",
    "tls_client_fingerprint_sha1",
    "tls_client_subject_name",
    "tls_client_issuer_name",
    "tls_client_serial",
    "backend_host",
    "backend_port",
    "method",
    "path",
    "path_without_query",
    "protocol_version",
]

if __name__ == '__main__':
    gentokenlookup(OPTIONS, 'SHRPX_OPTID_', value_type='char', comp_fun='util::strieq_l')
    gentokenlookup(LOGVARS, 'LogFragmentType::', value_type='char', comp_fun='util::strieq_l', return_type='LogFragmentType', fail_value='LogFragmentType::NONE')



#!/usr/bin/env python3

from gentokenlookup import gentokenlookup

HEADERS = [
    ':authority',
    ':method',
    ':path',
    ':scheme',
    ':status',
    ':host', # for spdy
    ':protocol',
    'expect',
    'host',
    'if-modified-since',
    "te",
    "cookie",
    "http2-settings",
    "server",
    "via",
    "forwarded",
    "x-forwarded-for",
    "x-forwarded-proto",
    "alt-svc",
    "content-length",
    "location",
    "trailer",
    "link",
    "accept-encoding",
    "accept-language",
    "cache-control",
    "user-agent",
    "date",
    "content-type",
    "early-data",
    "sec-websocket-accept",
    "sec-websocket-key",
    # disallowed h1 headers
    'connection',
    'keep-alive',
    'proxy-connection',
    'transfer-encoding',
    'upgrade'
]

if __name__ == '__main__':
    gentokenlookup(HEADERS, 'HD_')



#!/usr/bin/env python
"""End to end tests for the utility programs.

This test assumes the utilities inside src directory have already been
built.

At the moment top_buiddir is not in the environment, but top_builddir would be
more reliable than '..', so it's worth trying to pull it from the environment.
"""

__author__ = 'Jim Morrison <jim@twist.com>'


import os
import subprocess
import time
import unittest


_PORT = 9893


def _run_server(port, args):
  srcdir = os.environ.get('srcdir', '.')
  testdata = '%s/testdata' % srcdir
  top_builddir = os.environ.get('top_builddir', '..')
  base_args = ['%s/src/spdyd' % top_builddir, '-d', testdata]
  if args:
    base_args.extend(args)
  base_args.extend([str(port), '%s/privkey.pem' % testdata,
                    '%s/cacert.pem' % testdata])
  return subprocess.Popen(base_args)

def _check_server_up(port):
  # Check this check for now.
  time.sleep(1)

def _kill_server(server):
  while server.returncode is None:
    server.terminate()
    time.sleep(1)
    server.poll()


class EndToEndSpdyTests(unittest.TestCase):
  @classmethod
  def setUpClass(cls):
    cls.setUpServer([])

  @classmethod
  def setUpServer(cls, args):
    cls.server = _run_server(_PORT, args)
    _check_server_up(_PORT)

  @classmethod
  def tearDownClass(cls):
    _kill_server(cls.server)

  def setUp(self):
    build_dir = os.environ.get('top_builddir', '..')
    self.client = '%s/src/spdycat' % build_dir
    self.stdout = 'No output'

  def call(self, path, args):
     full_args = [self.client,'http://localhost:%d%s' % (_PORT, path)] + args
     p = subprocess.Popen(full_args, stdout=subprocess.PIPE,
                          stdin=subprocess.PIPE)
     self.stdout, self.stderr = p.communicate()
     return p.returncode


class EndToEndSpdy2Tests(EndToEndSpdyTests):
  def testSimpleRequest(self):
    self.assertEquals(0, self.call('/', []))

  def testSimpleRequestSpdy3(self):
    self.assertEquals(0, self.call('/', ['-v', '-3']))
    self.assertIn('NPN selected the protocol: spdy/3', self.stdout)

  def testFailedRequests(self):
    self.assertEquals(
        2, self.call('/', ['https://localhost:25/', 'http://localhost:79']))

  def testOneFailedRequest(self):
    self.assertEquals(1, subprocess.call([self.client, 'http://localhost:2/']))

  def testOneTimedOutRequest(self):
    self.assertEquals(1, self.call('/?spdyd_do_not_respond_to_req=yes',
                                   ['--timeout=2']))
    self.assertEquals(0, self.call('/', ['--timeout=20']))


class EndToEndSpdy3Tests(EndToEndSpdyTests):
  @classmethod
  def setUpClass(cls):
    cls.setUpServer(['-3'])

  def testSimpleRequest(self):
    self.assertEquals(0, self.call('/', ['-v']))
    self.assertIn('NPN selected the protocol: spdy/3', self.stdout)


if __name__ == '__main__':
  unittest.main()



#!/usr/bin/env python3

HEADERS = [
    (':authority', 0),
    (':method', 1),
    (':method', 2),
    (':path', 3),
    (':path', 4),
    (':scheme', 5),
    (':scheme', 6),
    (':status', 7),
    (':status', 8),
    (':status', 9),
    (':status', 10),
    (':status', 11),
    (':status', 12),
    (':status', 13),
    ('accept-charset', 14),
    ('accept-encoding', 15),
    ('accept-language', 16),
    ('accept-ranges', 17),
    ('accept', 18),
    ('access-control-allow-origin', 19),
    ('age', 20),
    ('allow', 21),
    ('authorization', 22),
    ('cache-control', 23),
    ('content-disposition', 24),
    ('content-encoding', 25),
    ('content-language', 26),
    ('content-length', 27),
    ('content-location', 28),
    ('content-range', 29),
    ('content-type', 30),
    ('cookie', 31),
    ('date', 32),
    ('etag', 33),
    ('expect', 34),
    ('expires', 35),
    ('from', 36),
    ('host', 37),
    ('if-match', 38),
    ('if-modified-since', 39),
    ('if-none-match', 40),
    ('if-range', 41),
    ('if-unmodified-since', 42),
    ('last-modified', 43),
    ('link', 44),
    ('location', 45),
    ('max-forwards', 46),
    ('proxy-authenticate', 47),
    ('proxy-authorization', 48),
    ('range', 49),
    ('referer', 50),
    ('refresh', 51),
    ('retry-after', 52),
    ('server', 53),
    ('set-cookie', 54),
    ('strict-transport-security', 55),
    ('transfer-encoding', 56),
    ('user-agent', 57),
    ('vary', 58),
    ('via', 59),
    ('www-authenticate', 60),
    ('te', None),
    ('connection', None),
    ('keep-alive',None),
    ('proxy-connection', None),
    ('upgrade', None),
    (':protocol', None),
    ('priority', None),
]

def to_enum_hd(k):
    res = 'NGHTTP2_TOKEN_'
    for c in k.upper():
        if c == ':' or c == '-':
            res += '_'
            continue
        res += c
    return res

def build_header(headers):
    res = {}
    for k, _ in headers:
        size = len(k)
        if size not in res:
            res[size] = {}
        ent = res[size]
        c = k[-1]
        if c not in ent:
            ent[c] = []
        if k not in ent[c]:
            ent[c].append(k)

    return res

def gen_enum():
    name = ''
    print('typedef enum {')
    for k, token in HEADERS:
        if token is None:
            print('  {},'.format(to_enum_hd(k)))
        else:
            if name != k:
                name = k
                print('  {} = {},'.format(to_enum_hd(k), token))
    print('} nghttp2_token;')

def gen_index_header():
    print('''\
static int32_t lookup_token(const uint8_t *name, size_t namelen) {
  switch (namelen) {''')
    b = build_header(HEADERS)
    for size in sorted(b.keys()):
        ents = b[size]
        print('''\
  case {}:'''.format(size))
        print('''\
    switch (name[{}]) {{'''.format(size - 1))
        for c in sorted(ents.keys()):
            headers = sorted(ents[c])
            print('''\
    case '{}':'''.format(c))
            for k in headers:
                print('''\
      if (memeq("{}", name, {})) {{
        return {};
      }}'''.format(k[:-1], size - 1, to_enum_hd(k)))
            print('''\
      break;''')
        print('''\
    }
    break;''')
    print('''\
  }
  return -1;
}''')

if __name__ == '__main__':
    gen_enum()
    print()
    gen_index_header()



#!/usr/bin/env python3
# -*- coding: utf-8 -*-

import subprocess
import io
import re
import sys
import os.path

class Option:
    def __init__(self, long_opt, short_opt):
        self.long_opt = long_opt
        self.short_opt = short_opt

def get_all_options(cmd):
    opt_pattern = re.compile(r'  (?:(-.), )?(--[^\s\[=]+)(\[)?')
    proc = subprocess.Popen([cmd, "--help"], stdout=subprocess.PIPE)
    stdoutdata, _ = proc.communicate()
    cur_option = None
    opts = {}
    for line in io.StringIO(stdoutdata.decode('utf-8')):
        match = opt_pattern.match(line)
        if not match:
            continue
        long_opt = match.group(2)
        short_opt = match.group(1)
        opts[long_opt] = Option(long_opt, short_opt)

    return opts

def output_case(out, name, opts):
    out.write('''\
_{name}()
{{
    local cur prev split=false
    COMPREPLY=()
    COMP_WORDBREAKS=${{COMP_WORDBREAKS//=}}

    cmd=${{COMP_WORDS[0]}}
    _get_comp_words_by_ref cur prev
'''.format(name=name))

    # Complete option name.
    out.write('''\
    case $cur in
        -*)
            COMPREPLY=( $( compgen -W '\
''')
    for opt in opts.values():
        out.write(opt.long_opt)
        out.write(' ')

    out.write('''\
' -- "$cur" ) )
            ;;
''')
    # If no option found for completion then complete with files.
    out.write('''\
        *)
            _filedir
            return 0
    esac
    return 0
}}
complete -F _{name} {name}
'''.format(name=name))

if __name__ == '__main__':
    if len(sys.argv) < 2:
        print("Generates bash_completion using `/path/to/cmd --help'")
        print("Usage: make_bash_completion.py /path/to/cmd")
        exit(1)
    name = os.path.basename(sys.argv[1])
    opts = get_all_options(sys.argv[1])
    output_case(sys.stdout, name, opts)



#!/usr/bin/env python3

def to_enum_hd(k, prefix):
    res = prefix
    for c in k.upper():
        if c == ':' or c == '-':
            res += '_'
            continue
        res += c
    return res

def build_header(headers):
    res = {}
    for k in headers:
        size = len(k)
        if size not in res:
            res[size] = {}
        ent = res[size]
        c = k[-1]
        if c not in ent:
            ent[c] = []
        ent[c].append(k)

    return res

def gen_enum(tokens, prefix):
    print('''\
enum {''')
    for k in sorted(tokens):
        print('''\
  {},'''.format(to_enum_hd(k, prefix)))
    print('''\
  {}MAXIDX,
}};'''.format(prefix))

def gen_index_header(tokens, prefix, value_type, comp_fun, return_type, fail_value):
    print('''\
{} lookup_token(const {} *name, size_t namelen) {{
  switch (namelen) {{'''.format(return_type, value_type))
    b = build_header(tokens)
    for size in sorted(b.keys()):
        ents = b[size]
        print('''\
  case {}:'''.format(size))
        print('''\
    switch (name[{}]) {{'''.format(size - 1))
        for c in sorted(ents.keys()):
            headers = sorted(ents[c])
            print('''\
    case '{}':'''.format(c))
            for k in headers:
                print('''\
      if ({}("{}", name, {})) {{
        return {};
      }}'''.format(comp_fun, k[:-1], size - 1, to_enum_hd(k, prefix)))
            print('''\
      break;''')
        print('''\
    }
    break;''')
    print('''\
  }}
  return {};
}}'''.format(fail_value))

def gentokenlookup(tokens, prefix, value_type='uint8_t', comp_fun='util::streq_l', return_type='int', fail_value='-1'):
    gen_enum(tokens, prefix)
    print()
    gen_index_header(tokens, prefix, value_type, comp_fun, return_type, fail_value)
